
#include <stdio.h>
#include "api.h"
#include "string.h"
#include "parameters.h"
#include <time.h>

#include "nist-rng.h"
#include <sys/syscall.h>

long long cpucycles() {
  unsigned long long result;
  __asm__ volatile(".byte 15;.byte 49;shlq $32,%%rdx;orq %%rdx,%%rax" : "=a" (result) ::  "%rdx");
  return result;
}

int main() {
  printf("\n");
  printf("*******************\n");
  printf("**** ROLLO-%d ****\n", PARAM_SECURITY);
  printf("*******************\n");

  printf("\n");
  printf("M: %d  |  ", PARAM_M);
  printf("N: %d  |  ", PARAM_N);
  printf("D: %d  |  ", PARAM_D);
  printf("R: %d  |  ", PARAM_R);
  printf("DFR: 2^-%d  |  ", PARAM_DFR);
  printf("Sec: %d bits", PARAM_SECURITY);
  printf("\n");

  unsigned char pk[PUBLIC_KEY_BYTES];
  unsigned char sk[SECRET_KEY_BYTES];

  unsigned char ct[CIPHERTEXT_BYTES];
  unsigned char ss1[SHARED_SECRET_BYTES];
  unsigned char ss2[SHARED_SECRET_BYTES];
  
  unsigned char seed[48];
  syscall(SYS_getrandom, seed, 48, 0);
  randombytes_init(seed, NULL, 256);

  clock_t t1 = clock();
  crypto_kem_keypair(pk, sk);
  clock_t t2 = clock();

  clock_t t3 = clock();
  crypto_kem_enc(ct, ss1, pk);
  clock_t t4 = clock();

  clock_t t5 = clock();
  crypto_kem_dec(ss2, ct, sk);
  clock_t t6 = clock();

  printf("\nKeygen: %.2f ms", (double) 1000 * (t2 - t1) / CLOCKS_PER_SEC);
  printf("\nEncaps: %.2f ms", (double) 1000 * (t4 - t3) / CLOCKS_PER_SEC);
  printf("\nDecaps: %.2f ms", (double) 1000 * (t6 - t5) / CLOCKS_PER_SEC);

  printf("\n\nsecret1: ");
  for(int i = 0 ; i < SHARED_SECRET_BYTES ; ++i) printf("%02x", ss1[i]);

  printf("\nsecret2: ");
  for(int i = 0 ; i < SHARED_SECRET_BYTES ; ++i) printf("%02x", ss2[i]);
  printf("\n\n");

  int result = 1;
  for(int i = 0 ; i < SHARED_SECRET_BYTES ; ++i) {
    if(ss1[i] != ss2[i]) {
      result = 0;
      break;
    }
  }

  printf("\nRESULT: %i\n", result);
}

